/*!
CTextureTGA.cpp
 Menu Disc Demo
 
 Author: Jeroen Verbeek
 Copyright (c) Jeroen Verbeek, 2003-2004. All rights reserved.
 
 This software is provided without any guarantee.
 No responsibility taken for any problems caused by the use if this software.
 
 Anyone is authorised to use this program code and methods without warranty,
 all that is required is that you contact me (jeroen@happymakinggames.com),
 and tell me that you use it and what you use it in -
 all I want is to know is who uses it and where.
 
 I will not ask for payment in any way: this code is free.
 If you encounter a bug with the code, please tell me about it.
 If you have a bug-fix for the code please provide me with it.
 If you think of an improvement please tell me about it.
 You are not required to provide me with bug reports, bug fixes or improvements.
 If you require me to perform any work related to this code or
 not related to this code, please contact me: I am always open to offers.
 
 Pie menus are in no way my original idea, I stumbled on the idea a few years ago,
 specifically Don Hopkins' http://catalog.com/hopkins/piemenus/piemenus-hopkins.html.
 I am a big fan of circular menus (disc menus).
 I am of the opinion that they provide the quickest method of selecting an action
 from a structured tree of actions. A menu of actions organised following this method
 will, in time, allow almost "blind selection" of actions. Each action selection
 requires the same distance of mouse movement, just in a slightly diferent angle.
 In my opinion this seems to work best with menu discs of no more than 8 items.
 There is plenty of research to be found through the internet regarding the 
 advantages of circular menus, for example: muscle memory.
 Some websites I recommend you look at are:
 
 http://catalog.com/hopkins/piemenus/piemenus-hopkins.html
 
 http://www.piemenus.com/
 
 http://www.sm.luth.se/~david/carl/www/argonaut.html
 
 Contact: Jeroen Verbeek, jeroen@happymakinggames.com, Sydney NSW Australia
 */
/*-----------------------------------------------------------------------------------------------*/
#include <stdio.h>
#include <GLut/glut.h>
#include "CLogger.h"
#include "CTexture.h"


/*-----------------------------------------------------------------------------------------------*/
/*!
Constructor.
 */
/*-----------------------------------------------------------------------------------------------*/
CTextureTGA::CTextureTGA()
{
	m_ImageDataKept = false;
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Free image data if it was kept.
 */
/*-----------------------------------------------------------------------------------------------*/
CTextureTGA::~CTextureTGA()
{
	//! If the imagedata was kept for some reason...
	if(m_ImageDataKept)
		free(m_ImageData);
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Reads in image data.
*/
/*-----------------------------------------------------------------------------------------------*/
unsigned char *CTextureTGA::ReadData(FILE *file)
{
    unsigned char *data = NULL;
    unsigned char temp;
    int bytesread, i;
	
	//! Calculate the image size, allocate a buffer and read the file.
	
	m_Size = m_ImageWidth * m_ImageHeight * m_ImagePixelSize;
    data = (unsigned char *)malloc(m_Size); 
	
    if(data == NULL)
        return 0;
	
    bytesread = fread(data, sizeof(unsigned char), m_Size, file);
	
    if(bytesread != m_Size)
    {
        free(data);
        return 0;
    }
	
	//! Process pixels differently depending on pixel bit depth.

	if(m_ImageDepth == 24 || m_ImageDepth == 32)
	{		
		//! If it's a 24 or 32 bit image then flip the RGB bytes:
		//! TGA is stored in BGR (or BGRA), make it RGB (or RGBA)...
		for(i = 0; i < m_Size; i += m_ImagePixelSize)
		{
			temp = data[i];
			data[i] = data[i + 2];
			data[i + 2] = temp;
		}
	}
	
	switch(m_ImageDepth)
	{
		case 32:	m_TextureFormat = GL_RGBA;	break;
		case 24:	m_TextureFormat = GL_RGB;	break;
		//! this needs to be passed in... texFormat = GL_LUMINANCE;
		case  8:	m_TextureFormat = GL_ALPHA;	break;
	}
	return data;
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Reads in TGA image data in 8, 24 or 32 bit uncompressed and binds it to a texture ID.
*/
/*-----------------------------------------------------------------------------------------------*/
int CTextureTGA::Load(char *name, int id, int texturemode, bool keepimagedata)
{
	TGA_HEADER header;
    FILE *file;
	int error = 0;
	
    if(!(file = fopen(name, "r")))
    {
		g_pGlobalLog->LogError("CTextureTGA::Load() Filename does not exist:");
	}
	else
	{
		fread(&header, sizeof(TGA_HEADER), 1, file);
		
		if(header.colourmap != 0 || (header.imagetype != 2 && header.imagetype != 3))
		{
			g_pGlobalLog->LogError("CTextureTGA::Load() TGA is compressed: %s", name);
		}
		else
		{
			//! TGA files are stored using Intel byte ordering...	
			m_ImageWidth = header.width[0] + header.width[1] * 256; 
			m_ImageHeight = header.height[0] + header.height[1] * 256;
			m_ImageDepth = header.depth; 
			m_ImagePixelSize = header.depth / 8;
			
			if(!IsPowerOf2(m_ImageWidth) || !IsPowerOf2(m_ImageHeight))
			{
				g_pGlobalLog->LogError("CTextureTGA::Load() Bad height or width in %s", name);
			}
			else
			{
				if(m_ImageDepth != 32 && m_ImageDepth != 24 && m_ImageDepth != 8)
				{
					g_pGlobalLog->LogError("CTextureTGA::Load() Bad bitdepth of %d in %s", m_ImageDepth, name);
				}
				else
				{
					
					m_ImageData = ReadData(file);
					
					if(m_ImageData == NULL)
					{
						g_pGlobalLog->LogError("CTextureTGA::Load() Bad data in %s", name);
					}
					else
					{
						glBindTexture(GL_TEXTURE_2D, id);
						glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
						glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
						glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
						glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, texturemode);
						glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_NEAREST);
						glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
						gluBuild2DMipmaps(GL_TEXTURE_2D, texturemode, m_ImageWidth, m_ImageHeight, texturemode, GL_UNSIGNED_BYTE, m_ImageData);
					}
					if(!keepimagedata)
						free(m_ImageData);
				}
			}
		}
		fclose (file);
	}
	return error;
}

